package com.qmth.themis.business.forkjoin;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.ExamStudentDtoImport;
import com.qmth.themis.business.entity.*;
import com.qmth.themis.business.enums.RoleEnum;
import com.qmth.themis.business.enums.TaskTypeExecEnum;
import com.qmth.themis.business.service.TBTaskHistoryService;
import com.qmth.themis.business.service.TBUserRoleService;
import com.qmth.themis.business.service.TEExamStudentService;
import com.qmth.themis.business.service.TEStudentService;
import com.qmth.themis.common.exception.BusinessException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.RecursiveTask;

public class ForkJoinTask extends RecursiveTask<List<TEExamStudent>> {
    private final static Logger log = LoggerFactory.getLogger(ForkJoinTask.class);

    private int threshold = 500, start, end;//阀值，开始，结束，类型
    private List<Object> examStudentDtoImportList;
    private Map<String, Object> map = null;

    public ForkJoinTask() {

    }

    public ForkJoinTask(int threshold, int start, int end, List<Object> examStudentDtoImportList, Map<String, Object> map) {
        this.threshold = threshold;
        this.start = start;
        this.end = end;
        this.examStudentDtoImportList = examStudentDtoImportList;
        this.map = map;
    }

    @Override
    @Transactional
    protected List<TEExamStudent> compute() {
        Gson gson = new Gson();
        List<TEExamStudent> teExamStudentList = new ArrayList<>();
        Map tbTaskHistoryMap = (Map) map.get("tbTaskHistory");
        tbTaskHistoryMap = SystemConstant.timeTransform(tbTaskHistoryMap);
        TBTaskHistory tbTaskHistory = gson.fromJson(gson.toJson(tbTaskHistoryMap), TBTaskHistory.class);
        Long examId = Long.parseLong(String.valueOf(map.get("examId")));
        Long orgId = Long.parseLong(String.valueOf(map.get("orgId")));
        Long createId = Long.parseLong(String.valueOf(map.get("createId")));
        Map<String, TEExamActivity> teExamActivityMap = (Map<String, TEExamActivity>) map.get("teExamActivityMap");
        if ((end - start) <= threshold) {
            TEExamStudentService teExamStudentService = SpringContextHolder.getBean(TEExamStudentService.class);
            TBTaskHistoryService tbTaskHistoryService = SpringContextHolder.getBean(TBTaskHistoryService.class);
            TEStudentService teStudentService = SpringContextHolder.getBean(TEStudentService.class);
            TBUserRoleService tbUserRoleService = SpringContextHolder.getBean(TBUserRoleService.class);
            for (int i = start; i <= end; i++) {
                ExamStudentDtoImport examStudentDtoImport = (ExamStudentDtoImport) examStudentDtoImportList.get(i);
                Map m = (Map) teExamActivityMap.get(examStudentDtoImport.getExamActivityCode());
                m = SystemConstant.timeTransform(m);
                TEExamActivity teExamActivity = gson.fromJson(gson.toJson(m), TEExamActivity.class);
                if (Objects.isNull(teExamActivity)) {
                    throw new BusinessException("没有" + examStudentDtoImport.getExamActivityCode() + "的考场");
                }
                //先根据证件号+科目代码查询考生是否存在，存在则更新，不存在则插入
                QueryWrapper<TEExamStudent> teExamStudentQueryWrapper = new QueryWrapper<>();
                teExamStudentQueryWrapper.lambda().eq(TEExamStudent::getIdentity, examStudentDtoImport.getIdentity()).eq(TEExamStudent::getCourseCode, examStudentDtoImport.getCourseCode());
                TEExamStudent teExamStudent = teExamStudentService.getOne(teExamStudentQueryWrapper);
                //如果为空则插入考生数据，插入考生前先插入学生档案数据
                if (Objects.isNull(teExamStudent)) {
                    //先插入学生档案数据
                    TEStudent teStudent = new TEStudent(orgId, examStudentDtoImport.getIdentity(), examStudentDtoImport.getName(), createId);
                    teStudentService.save(teStudent);

                    //插入用户角色关系
                    TBUserRole tbUserRole = new TBUserRole(teStudent.getId(), RoleEnum.STUDENT.name());
                    tbUserRoleService.save(tbUserRole);

                    teExamStudent = gson.fromJson(gson.toJson(examStudentDtoImport), TEExamStudent.class);
                    teExamStudent.setExamId(examId);
                    teExamStudent.setExamActivityId(teExamActivity.getId());
                    teExamStudent.setStudentId(teStudent.getId());
                    teExamStudent.setCreateId(createId);
                } else {
                    teExamStudent.setUpdateId(createId);
                    teExamStudent.setName(examStudentDtoImport.getName());
                    teExamStudent.setCourseName(examStudentDtoImport.getCourseName());
                    teExamStudent.setGrade(examStudentDtoImport.getGrade());
                    teExamStudent.setClassNo(examStudentDtoImport.getClassNo());
                    teExamStudent.setRoomCode(examStudentDtoImport.getRoomCode());
                    teExamStudent.setRoomName(examStudentDtoImport.getRoomName());
                    teExamStudent.setExamActivityId(teExamActivity.getId());
                }
                teExamStudentList.add(teExamStudent);
            }
            BigDecimal bigDecimal = new BigDecimal(100);
            int size = examStudentDtoImportList.size();
            BigDecimal endBigDecimal = new BigDecimal(teExamStudentList.size());
            BigDecimal progress = new BigDecimal(Double.valueOf(endBigDecimal.divide(new BigDecimal(size), 2, BigDecimal.ROUND_HALF_UP).multiply(bigDecimal).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue())).setScale(0, BigDecimal.ROUND_HALF_UP);
            teExamStudentService.saveOrUpdateBatch(teExamStudentList);
            if (progress.intValue() == 100) {
                tbTaskHistory.setStatus(TaskTypeExecEnum.finish.ordinal());
                tbTaskHistory.setSummary("共处理了" + size + "条数据");
                tbTaskHistory.setFinishTime(new Date());
            } else {
                tbTaskHistory.setStatus(TaskTypeExecEnum.running.ordinal());
                tbTaskHistory.setSummary("正在处理第" + start + "条至" + end + "条数据");
            }
            tbTaskHistory.setProgress(progress.doubleValue());
            tbTaskHistoryService.updateById(tbTaskHistory);
            log.info("start:{},end:{}", start, end);
        } else {
            // 将大任务分解成两个小任务
            int middle = (start + end) / 2;
            ForkJoinTask left = new ForkJoinTask(threshold, start, middle, examStudentDtoImportList, map);
            ForkJoinTask right = new ForkJoinTask(threshold, middle + 1, end, examStudentDtoImportList, map);
            invokeAll(left, right);
            teExamStudentList.addAll(left.join());
            teExamStudentList.addAll(right.join());
        }
        return teExamStudentList;
    }
}
