package com.qmth.themis.business.templete.impl;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.dto.ExamStudentDtoImport;
import com.qmth.themis.business.entity.TBExamInvigilateUser;
import com.qmth.themis.business.entity.TEExamStudent;
import com.qmth.themis.business.enums.ExamModeEnum;
import com.qmth.themis.business.forkjoin.ForkJoinTask;
import com.qmth.themis.business.service.TBExamInvigilateUserService;
import com.qmth.themis.business.templete.TaskImportTemplete;
import com.qmth.themis.business.util.ExcelCallback;
import com.qmth.themis.business.util.ExcelError;
import com.qmth.themis.business.util.ExcelUtil;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.LinkedMultiValueMap;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.Future;

/**
 * @Description: 考生导入模版
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/15
 */
public class ExamStudentTemplete implements TaskImportTemplete {
    private final static Logger log = LoggerFactory.getLogger(ExamStudentTemplete.class);

    @Override
    public Result importTask(File file, Map<String, Object> map) throws IOException, InterruptedException {
        Integer mode = Integer.parseInt(String.valueOf(map.get("mode")));
        Long orgId = Long.parseLong(String.valueOf(map.get("orgId")));
        Long createId = Long.parseLong(String.valueOf(map.get("createId")));
        FileInputStream inputStream = new FileInputStream(file);
        List<LinkedMultiValueMap<Integer, Object>> finalList = ExcelUtil.excelReader(inputStream, Lists.newArrayList(ExamStudentDtoImport.class), new ExcelCallback() {
            @Override
            public List<LinkedMultiValueMap<Integer, Object>> callback(List<LinkedMultiValueMap<Integer, Object>> finalList, List<LinkedMultiValueMap<Integer, String>> finalColumnNameList) throws IllegalAccessException {
                List<ExcelError> excelErrorList = new ArrayList<>();
                for (int i = 0; i < finalList.size(); i++) {
                    LinkedMultiValueMap<Integer, Object> map = finalList.get(i);
                    List<Object> examStudentDtoImportList = map.get(i);
                    log.info("examStudentDtoImportList.size:{}", examStudentDtoImportList.size());
                    for (int y = 0; y < examStudentDtoImportList.size(); y++) {
                        ExamStudentDtoImport examStudentDtoImport = (ExamStudentDtoImport) examStudentDtoImportList.get(y);
                        List<ExcelError> excelErrorTemp = ExcelUtil.checkExcelField(examStudentDtoImport, y, i);
                        if (excelErrorTemp.size() > 0) {
                            excelErrorList.addAll(excelErrorTemp);
                        }
                        //校验考试模式，如果是集中统一，则需填写考试场次
                        if (mode.intValue() == ExamModeEnum.together.ordinal() && Objects.isNull(examStudentDtoImport.getExamActivityCode())) {
                            excelErrorList.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行考场为空"));
                        }
                    }
                }
                if (excelErrorList.size() > 0) {
                    throw new BusinessException(JSONObject.toJSONString(excelErrorList));
                }
                return finalList;
            }
        });
        //保存数据到数据库
        if (Objects.nonNull(finalList) && finalList.size() > 0) {
            log.info("开始forkjoin数据拆分");
            long start = System.currentTimeMillis();
            ForkJoinPool forkJoinPool = new ForkJoinPool();
            for (int i = 0; i < finalList.size(); i++) {
                LinkedMultiValueMap<Integer, Object> multiValueMap = finalList.get(i);
                List<Object> examStudentDtoImportList = multiValueMap.get(i);
                ForkJoinTask forkJoinTask = new ForkJoinTask(500, 0, examStudentDtoImportList.size() - 1, examStudentDtoImportList, map);
                Future<List<TEExamStudent>> result = forkJoinPool.submit(forkJoinTask);
                if (forkJoinTask.isCompletedAbnormally()) {
                    //检查任务是否已经抛出异常或已经被取消了，要注意此方法。由于提交任务之后，检测该任务是否有异常，不是阻塞的。所以需要上面的等待任务的完成。才能正确的获取到是否有异常
                    throw new BusinessException(forkJoinTask.getException().getMessage());
                }
                try {
                    List<TEExamStudent> list = result.get();
                    Map<String, String> tbExamInvigilateUserMap = (Map<String, String>) map.get("tbExamInvigilateUserMap");
                    if (Objects.nonNull(list) && list.size() > 0) {
                        //考场的创建
                        TBExamInvigilateUserService tbExamInvigilateUserService = SpringContextHolder.getBean(TBExamInvigilateUserService.class);
                        Set<String> roomCodeAndNameSet = new HashSet<>();
                        list.forEach(s -> {
                            roomCodeAndNameSet.add(s.getRoomCode() + ":" + s.getRoomName());
                        });
                        List<TBExamInvigilateUser> tbExamInvigilateUserList = new ArrayList<>();
                        roomCodeAndNameSet.forEach(s -> {
                            if (Objects.isNull(tbExamInvigilateUserMap) || (Objects.nonNull(tbExamInvigilateUserMap) && Objects.isNull(tbExamInvigilateUserMap.get(s)))) {
                                String[] strs = s.split(":");
                                TBExamInvigilateUser tbExamInvigilateUser = new TBExamInvigilateUser(orgId, createId, strs[0], strs[1]);
                                tbExamInvigilateUserList.add(tbExamInvigilateUser);
                            }
                        });
                        tbExamInvigilateUserService.saveBatch(tbExamInvigilateUserList);
                    }
                    log.info("list.size:{}", list.size());
                } catch (ExecutionException | InterruptedException e) {
                    e.printStackTrace();
                } finally {
                    if (Objects.nonNull(forkJoinPool)) {
                        forkJoinPool.shutdown();
                    }
                }
            }
            long end = System.currentTimeMillis();
            log.info("forkjoin数据拆分结束,============耗时============:{}秒", (end - start) / 1000);
        }
        Map resultMap = new HashMap<>();
        return ResultUtil.ok(resultMap);
    }
}
